//=========================================================================
//Copyright LizardOfOz.
//This is a VERY early version intended to prototype maps.
//Expect things to change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy, Velly, JPRAS - Saxton Hale's model imitating the Jungle Inferno short.
//  MegapiemanPHD - Hale's voice lines.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

isRoundSetup <- true
starterClass <- {}
damage <- {}
isFeedbackRound <- false;

function StartNewRound(void)
{
    if (IsInWaitingForPlayers())
        return;

    isRoundSetup = true;

    local firstPlayer = Entities.FindByClassname(null, "player")
    local secondPlayer = Entities.FindByClassname(firstPlayer, "player")
    if (IsNotValidPlayer(firstPlayer) && IsNotValidPlayer(secondPlayer))
        return;
    ProgressHaleQueue();

    local player = null;
    while (player = Entities.FindByClassname(player, "player"))
    {
        if (IsNotHale(player) && IsValidPlayer(player))
        {
            player.ForceChangeTeam(TF_TEAM.MERCS, true);
            player.KeyValueFromString("targetname", "");
            TrySpawnAsMerc(player);
        }
    }

    TrySpawnAsHale(GetHale());
    DoEntFire("logic_script_vsh", "RunScriptCode", "PlaySoundForAll(`VSH.Start`)", 3, null, null);
}
AddListener("start", StartNewRound, 1);

function SetRoundSetup(isSetup)
{
    isRoundSetup = isSetup;
    TrySpawnAsHale(GetHale());

    if (isSetup)
        return;

    local player = null;
    while (player = Entities.FindByClassname(player, "player"))
        if (IsValidPlayer(player) && IsNotHale(player))
            starterClass[player] <- player.GetPlayerClass();

    local respawnRoom = null;
    while (respawnRoom = Entities.FindByClassname(respawnRoom, "func_respawnroom"))
        respawnRoom.Kill();
}

function ProgressHaleQueue()
{
    local gamerules = Entities.FindByClassname(null, "tf_gamerules");
    if (NetProps.GetPropInt(gamerules, "m_nHudType") == 0)
        NetProps.SetPropInt(gamerules, "m_nHudType", 2);
    gamerules.ValidateScriptScope();
    local globalVars = gamerules.GetScriptScope();

    if ("nextHale" in globalVars && IsValidPlayer(globalVars.nextHale))
    {
        SetHale(globalVars.nextHale);
        globalVars.lastHale <- GetHale();
    }
    else
    {
        local newHale = RollNewHale("lastHale" in globalVars ? globalVars.lastHale : null);
        globalVars.lastHale <- newHale;
        SetHale(newHale);
    }
    globalVars.nextHale <- null;
}

function RollNewHale(ignoredPlayer)
{
    local players = [];
    local player = null;
    while (player = Entities.FindByClassname(player, "player"))
    {
        if (player != ignoredPlayer && IsValidPlayer(player))
            players.push(player);
    }
    if (players.len() > 0)
        return players[RandomInt(0, players.len() - 1)];
    local theOnlyPlayer = Entities.FindByClassname(null, "player");
    return (theOnlyPlayer != null && !theOnlyPlayer.IsFakeClient()) ? theOnlyPlayer : null ;
}

roundWon <- false;

function EndRound(winner)
{
    if (roundWon)
        return;
    roundWon = true;
    local roundWin = Entities.FindByClassname(null, "game_round_win");
    if (roundWin == null)
    {
        roundWin = SpawnEntityFromTable("game_round_win",
        {
            win_reason = "0",
            force_map_reset = "1", //not having
            TeamNum = "0",         //these 3 lines
            switch_teams = "0"     //causes the crash when trying to fire game_round_win
        });
    }
    EntFireByHandle(roundWin, "SetTeam", "" + winner, 0, null, null);
    EntFireByHandle(roundWin, "RoundWin", "", 0, null, null);
    PlaySoundForAll(winner == TF_TEAM.HALE ? "VSH.HaleWins" : "VSH.HaleLoses");
}
