//=========================================================================
//Copyright LizardOfOz.
//This is a VERY early version intended to prototype maps.
//Expect things to change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy, Velly, JPRAS - Saxton Hale's model imitating the Jungle Inferno short.
//  MegapiemanPHD - Hale's voice lines.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

function HaleTookDamageEvent(params)
{
    local hale = params.const_entity;
    local attacker = params.attacker;
    local inflictor = params.inflictor;

    if (IsNotHale(hale))
        return;

    if (IsValidPlayer(attacker))
    {
        //Le backstabe
        if (params.damage_custom == Constants.ETFDmgCustom.TF_DMG_CUSTOM_BACKSTAB)
        {
            params.damage = CalcStabDamage() / 2.5; //Crit compensation

            NetProps.SetPropFloat(params.weapon, "m_flNextPrimaryAttack", Time() + 2.0);
            NetProps.SetPropFloat(attacker, "m_flNextAttack", Time() + 2.0);
            NetProps.SetPropFloat(attacker, "m_flStealthNextChangeTime", Time() + 2.0);
            EmitSoundOn("Player.Spy_Shield_Break", hale);
            EmitSoundOn("Player.Spy_Shield_Break", hale);
        }

        //Miniguns (nerf damage by 50%)
        if (params.weapon != null && params.weapon.GetClassname() == "tf_weapon_minigun")
            params.damage *= 0.5

        //Holiday Punch (tracking the taunt stun)
        if (params.weapon != null && params.weapon.GetWeaponId() == 656)
            lastHolidayPunchTime = Time();

        //Sentry Gun (`SET BONUS: dmg from sentry reduce` attribute doesn't reduce knockback)
        if (IsValidBuilding(inflictor))
            params.damage *= 0.3;
    }

    //Death pit
    if (inflictor.GetClassname() == "trigger_hurt" && params.damage >= 1000)
        params.damage = CalcDeathPitDamage();

    AddDamageToRageMeter(params.damage)
}
AddListener("damage", HaleTookDamageEvent, 1);

function SaxtonPunchEvent(params)
{
    local hale = params.attacker;
    local victim = params.const_entity;
    if (IsNotHale(hale) || !IsValidPlayerOrBuilding(victim) || halePunchMeter < 0 || params.damage_type == 1)
        return;

    halePunchMeter -= 30;

    local haleEyeVector = hale.EyeAngles().Forward();
    haleEyeVector.Norm();

    foreach(player in GetAliveMercs())
    {
        local posDiff = player.GetCenter() - hale.GetCenter();
        local distance = posDiff.Norm();
        if (distance > 400)
            continue;

        local dot = haleEyeVector.Dot(posDiff);
        if (dot < 0.6)
            continue;

        local distanceToDamage = distance < 100 ? 1 : 2.0 / sqrt(distance);
        local distanceToPush = distance < 100 ? 10 : 10 / sqrt(distance);

        NetProps.SetPropEntity(player, "m_hGroundEntity", null);
        posDiff.x = posDiff.x * 1250 * distanceToPush;
        posDiff.y = posDiff.y * 1250 * distanceToPush;
        posDiff.z = 750 * distanceToPush;
        player.ApplyAbsVelocityImpulse(posDiff);
        player.TakeDamageEx(hale, hale, hale.GetActiveWeapon(), posDiff, player.GetCenter(), player.GetMaxHealth() * distanceToDamage, 1);
        DispatchParticleEffect("mvm_pow_punch", victim.EyePosition() + Vector(0,0,30), Vector(0,0,0));
    }

    local building = null;
    while (building = Entities.FindByClassname(building, "obj_*"))
    {
        local posDiff = building.GetCenter() - hale.GetCenter();
        local distance = posDiff.Norm();
        if (distance > 400)
            continue;

        local distanceToDamage = distance < 100 ? 1 : 2.0 / sqrt(distance);

        building.TakeDamageEx(hale, hale, hale.GetActiveWeapon(), Vector(0,0,0), building.GetCenter(), building.GetMaxHealth() * distanceToDamage, 1);
    }

    NetProps.SetPropEntity(hale, "m_hActiveWeapon", hale.GiveWeapon("Hale's Own Fists"));
    EmitSoundOn("TFPlayer.CritHit", hale);
    EmitSoundOn("VSH.Punch", hale);
    DispatchParticleEffect("passtime_air_blast", victim.GetCenter(), haleEyeVector);
    DispatchParticleEffect("Explosion_ShockWave_01", victim.GetOrigin(), Vector(0,0,0));
    ScreenShake(hale.GetCenter(), 10, 2.5, 1, 1000, 0, true);
}
AddListener("damage", SaxtonPunchEvent, 0);